﻿
#include "FpDriver.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

int SaveFinger(const char* fileName, const char *finger)
{
  FILE *fp;

  fp = fopen(fileName,"wt");
  if(!fp)
  {
    return -2;
  }
  fwrite(finger, 1, strlen(finger), fp);
  fclose(fp);
  return 0;
}

int LoadFinger(const char* fileName, char* finger)
{
  FILE *fp;
  int length;

  fp = fopen(fileName,"rt");
  if(!fp)
  {
    return -2;
  }
  length = fread(finger, 1, 685, fp);
  finger[length] = 0;
  fclose(fp);
  return 0;
}

int LoadImage(const char* fileName, char* image)
{
  FILE *fp;

  fp = fopen(fileName,"rt");
  if(!fp)
  {
    return -2;
  }
  fread(image, 1, 30*1024, fp);
  fclose(fp);
  return 0;
}

int LoadData(const char* fileName, char* data)
{
  FILE *fp;

  fp = fopen(fileName,"rt");
  if(!fp)
  {
    return -2;
  }
  fread(data, 1, 30*1024, fp);
  fclose(fp);
  return 0;
}

void GetDeviceInfo()
{
  char szFactoryID[10] = {0};
  char szDevTypeID[10] = {0};
  char szFactory[50] = {0};
  char szDevMode[100] = {0};
  char szFirmVer[100] = {0};
  char szDeviceID[50] = {0};
  int result;

  result = FPIGetDeviceInfo(0, szFactoryID, szDevTypeID,
    szFactory, szDevMode, szFirmVer);
  if (result == 0)
  {
    printf("厂商ID:%s,设备类型ID:%s,厂家信息:%s设备型号:%s,固件版本:%s\n",
      szFactoryID, szDevTypeID, szFactory, szDevMode, szFirmVer);
  }
  else
  {
    printf("获取设备信息错误:%d\n", result);
  }

  result = FPIGetDeviceID(0, szDeviceID);
  if (result == 0)
  {
    printf("设备序列号:%s\n", szDeviceID);
  }
  else
  {
    printf("获取设备序列号错误:%d\n", result);
  }
}

void InitKey()
{
  int result;

  result = FPIDeviceFormat(0);
  if (result == 0)
  {
    printf("清空密钥成功.\n");
  }
  else
  {
    printf("清空密钥错误:%d\n", result);
  }
}

#define TEST_DFK "SUNCQ19NSUFYSVNfVjEuMA=="
#define TEST_DMK "Xps1ok88dnJEKU1OocnUbBcLJB3xsfaL"
#define TEST_DEK "Xps1ok88dnJEKU1OocnUbBcLJB3xsfaL"
#define TEST_DAK "Xps1ok88dnJEKU1OocnUbBcLJB3xsfaL"
#define TEST_DMK_MAC "HLNN7RY4M+Q="
#define TEST_DEK_MAC "HLNN7RY4M+Q="
#define TEST_DAK_MAC "HLNN7RY4M+Q="
#define TEST_RANDOM "AAAAAAAAAAA="

void LoadKey()
{
  int result;

  result = FPISetDFK(0, "", "DFK", "001", TEST_DFK);
  if (result == 0)
  {
    printf("设置出厂密钥成功.\n");
  }
  else
  {
    printf("设置出厂密钥错误:%d\n", result);
    return;
  }

  result = FPISetDMK(0, "", "DMK", "001", TEST_DMK, TEST_DMK_MAC);
  if (result == 0)
  {
    printf("设置主密钥成功.\n");
  }
  else
  {
    printf("设置主密钥错误:%d\n", result);
    return;
  }
  result = FPISetWorkKey(0, "", 2, "DEK", "001", TEST_DEK, TEST_DEK_MAC); 
  if (result == 0)
  {
    printf("设置加密密钥成功.\n");
  }
  else
  {
    printf("设置加密密钥错误:%d\n", result);
    return;
  }
  result = FPISetWorkKey(0, "", 3, "DAK", "001", TEST_DAK, TEST_DAK_MAC);     
  if (result == 0)
  {
    printf("设置MAC密钥成功.\n");
  }
  else
  {
    printf("设置MAC密钥错误:%d\n", result);
    return;
  }
}

void GetKeyInfo()
{
  int result;
  char szKeyName[50] = {0};
  char szKeyVer[50] = {0};
 
  memset(szKeyName, 0, 50);
  memset(szKeyVer, 0, 50);
  result = FPIGetKeyInfo(0, "", 0, szKeyName, szKeyVer);
  if (result == 0)
  {
    printf("出厂密钥信息:名称:%s,版本:%s\n", szKeyName, szKeyVer);
  }
  else
  {
    printf("获取出厂密钥信息错误:%d\n", result);
  }
  
  memset(szKeyName, 0, 50);
  memset(szKeyVer, 0, 50);
  result = FPIGetKeyInfo(0, "", 0, szKeyName, szKeyVer);
  if (result == 0)
  {
    printf("主密钥信息:名称:%s,版本:%s\n", szKeyName, szKeyVer);
  }
  else
  {
    printf("获取主密钥信息错误:%d\n", result);
  }
  
  memset(szKeyName, 0, 50);
  memset(szKeyVer, 0, 50);
  result = FPIGetKeyInfo(0, "", 0, szKeyName, szKeyVer);
  if (result == 0)
  {
    printf("加密密钥信息:名称:%s,版本:%s\n", szKeyName, szKeyVer);
  }
  else
  {
    printf("获取加密密钥信息错误:%d\n", result);
  }

  memset(szKeyName, 0, 50);
  memset(szKeyVer, 0, 50);
  result = FPIGetKeyInfo(0, "", 0, szKeyName, szKeyVer);
  if (result == 0)
  {
    printf("MAC密钥信息:名称:%s,版本:%s\n", szKeyName, szKeyVer);
  }
  else
  {
    printf("获取MAC密钥信息错误:%d\n", result);
  }
}

void ReadFingerImage()
{
  int result;
  char szImg[50*1024] = {0};
  char szMac[50] = {0};
  int nLength;
  int nValid;

  result = FPIGetImage(0, "", TEST_RANDOM, szImg,
    &nLength, szMac, &nValid);
  if (result == 0)
  {
    printf("获取指纹图像成功.\n");
    printf("图像:%s\n", szImg);
    printf("MAC:%s\n", szMac);
    printf("Valid:%d\n", nValid);
  }
  else
  {
    printf("获取指纹图像错误:%d\n", result);
  }
}

int readOneImage(int flag, char* lpImage)
{
  int result;
  char szImg[50*1024] = {0};
  char szMac[50] = {0};
  int nLength;
  int nValid;

  while (1)
  {
    memset(szImg, 0, sizeof(szImg));
    result = FPIGetImage(0, "", TEST_RANDOM, szImg,
      &nLength, szMac, &nValid);
    if (result != 0)
    {
      return -1;
    }
    if (nValid == flag)
    {
      break;
    }
  }

  strcpy(lpImage, szImg);
  return 0;
}

void RegisterFinger()
{
  char szImg[3][50*1024];
  char szImg2[50*1024];
  char szTemp[512] = {0};
  int result;
  int i, nLength;

  for (i=0; i<3; i++)
  {
    printf("请按第%d次指纹\n", i+1);
    result = readOneImage(0, szImg[i]);
    if (result != 0)
    {
      printf("获取指纹图像错误\n");
      return;
    }
    printf("请拿开手指\n");
    readOneImage(1, szImg2);
  }

  result = FPIGetTemplateByImg(szImg[0], szImg[1],
    szImg[2], szTemp, &nLength);
  if (result == 0)
  {
    printf("获取指纹模板成功.\n");
    printf("模板:%s\n", szTemp);
    SaveFinger("temp.txt", szTemp);
  }
  else
  {
    printf("获取指纹模板错误:%d\n", result);
  }
}

void ReadFinger()
{
  int result;
  char szMac[50] = {0};
  char szFeat[512] = {0}; 
  result = FPIGetFeature(0, 10, "", TEST_RANDOM, szMac, szFeat);
  if (result == 0)
  {
    printf("获取指纹特征成功.\n");
    printf("特征:%s\n", szFeat);
    printf("MAC:%s\n", szMac);
    SaveFinger("feat.txt", szFeat);
  }
  else
  {
    printf("获取指纹特征错误:%d\n", result);
  }
}

void GetDek(unsigned char dekey[16])
{
  unsigned char dfk[24];
  unsigned char dmk[24];
  unsigned char dek[24];
  unsigned char key[24];
  int length;

  B64Decode(TEST_DFK, strlen(TEST_DFK), dfk);
  B64Decode(TEST_DMK, strlen(TEST_DMK), dmk);
  B64Decode(TEST_DEK, strlen(TEST_DEK), dek);

  des3_decrypt_data(dfk, dmk, 24, key, &length);
  des3_decrypt_data(key, dek, 24, dekey, &length);
}

void CompareFinger()
{
  char szFeat[512] = {0};
  char szTemp[512] = {0};
  unsigned char tzCryData[512];
  unsigned char tzData[512];
  unsigned char key[24];
  int length;
  int result;

  GetDek(key);

  LoadFinger("feat.txt", szFeat);
  LoadFinger("temp.txt", szTemp);
  length = B64Decode(szFeat, strlen(szFeat), tzCryData);
  des3_decrypt_data(key, tzCryData, length, tzData, &length);
  length = B64Encode(tzData, length, szFeat, 512);
  szFeat[length] = 0;
  printf("feat:%s\n", szFeat);
  printf("temp:%s\n", szTemp);

 result = FPIFpVerify(szTemp, szFeat, 3);
 if (result == 0)
 {
   printf("指纹比对成功\n");
 }
 else
 {
   printf("指纹不匹配\n");
 }
}

void UpdateDev()
{
  int result;
  result = FPIDeviceUpdate(0, "update.cod");
  if (result == 0)
  {
    printf("设备升级成功.\n");
  }
  else
  {
    printf("设备升级错误:%d\n", result);
  }
}

int main(int argc,char *argv[])
{
	int lRV=-1;
	int flag = 1;
	char szAct[100];
	while(flag)
	{
		printf("\n***************************************");
		printf("\n*    Miaxis function test             *");
		printf("\n*                                     *");
		printf("\n*      1. Get device info             *");
		printf("\n*      2. DownLoad  Key               *"); 
		printf("\n*      3. Init  Key                   *"); 
		printf("\n*      4. Get Key Info                *"); 
		printf("\n*      5. Get Finger Image            *"); 
		printf("\n*      6. Register Finger             *"); 
		printf("\n*      7. Read Finger                 *"); 
		printf("\n*      8. Match Finger                *"); 
		printf("\n*      9. Update Device               *"); 
		printf("\n*      0. quit                        *"); 
		printf("\n*                                     *");
		printf("\n***************************************");  
		printf("\n      select(0-9):");
		fgets(szAct, 10, stdin);
		int mode = atoi(szAct);
		switch(mode)
		{	
		case 1:
      GetDeviceInfo();
			break;
    case 2:
      LoadKey();
			break;
    case 3:
      InitKey();
      break;
    case 4:
      GetKeyInfo();
			break;
    case 5:
      ReadFingerImage();
			break;
		case 6:
      RegisterFinger();
			break;
		case 7:
      ReadFinger();
			break;
		case 8:
      CompareFinger();
			break;
		case 9:
      UpdateDev();
			break;
		case 0:
			flag = 0;
			break;
		}
	}
	return 0;
}

